/*
 * Copyright (c) 2017 - 2019 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/smack.h>
#include <linux/limits.h>
#include <glib.h>
#include <system_info.h>

#include "syspopup_private.h"
#include "syspopup_parser_db.h"
#include "syspopup_parser.h"
#include "syspopup_type.h"

#define PATH_SYSPOPUP_DEFAULT "/usr/share/syspopup/default"

#define PATH_SYSPOPUP "/usr/share/syspopup"

#define ROOT_UID 0

static int __load_syspopup_info(const char *path, GList **syspopup_list)
{
	struct dirent *dentry = NULL;
	char buf[PATH_MAX];
	char *ext;
	DIR *dp;

	if (!path)
		return -1;

	dp = opendir(path);
	if (!dp) {
		printf("Failed to open directory(%s). errno(%d)\n",
				path, errno);
		return -1;
	}

	while ((dentry = readdir(dp)) != NULL) {
		if (dentry->d_name[0] == '.')
			continue;

		ext = strrchr(dentry->d_name, '.');
		if (ext && strcmp(ext, ".xml") == 0) {
			snprintf(buf, sizeof(buf), "%s/%s",
					path, dentry->d_name);
			syspopup_parser_parse(buf, syspopup_list);
		}
	}
	closedir(dp);

	return 0;
}

static const char *__get_syspopup_path(void)
{
	static char path[PATH_MAX];
	char *profile = NULL;

	system_info_get_platform_string("http://tizen.org/feature/profile",
			&profile);
	if (profile == NULL) {
		printf("Failed to get profile info\n");
		snprintf(path, sizeof(path), "%s/wearable", PATH_SYSPOPUP);
		return path;
	}

	snprintf(path, sizeof(path), "%s/%s", PATH_SYSPOPUP, profile);
	free(profile);

	return path;
}

static int __init_db(void)
{
	GList *list = NULL;
	int ret;

	__load_syspopup_info(PATH_SYSPOPUP_DEFAULT, &list);
	__load_syspopup_info(__get_syspopup_path(), &list);
	if (!list)
		return -1;

	ret = syspopup_parser_db_open();
	if (ret < 0)
		goto end;

	ret = syspopup_parser_db_insert(list);

end:
	syspopup_parser_db_close();

	if (list)
		g_list_free_full(list, (GDestroyNotify)syspopup_free);

	return ret;
}

int main(int argc, char **argv)
{
	int ret;
	GOptionContext *context;
	GError *error = NULL;
	gpointer opt_force = NULL;
	GOptionEntry entries[] = {
		{"force", 'f', 0, G_OPTION_ARG_NONE, &opt_force,
			"Remove an existing db file and initialize it", NULL},
		{NULL}
	};

	if (getuid() != ROOT_UID) {
		printf("This binary should be run as root user\n");
		return -1;
	}

	context = g_option_context_new(NULL);
	g_option_context_add_main_entries(context, entries, NULL);
	if (!g_option_context_parse(context, &argc, &argv, &error)) {
		printf("%s: %s\n", argv[0], error->message);
		g_option_context_free(context);
		g_clear_error(&error);
		return -1;
	}
	g_option_context_free(context);

	if (opt_force) {
		ret = syspopup_parser_db_unlink();
		if (ret < 0)
			return -1;
	} else {
		if (syspopup_parser_db_exist()) {
			printf("The db file already exists.\n");
			return 0;
		}
	}

	return __init_db();
}
