/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/smack.h>
#include <errno.h>

#include <glib.h>
#include <sqlite3.h>
#include <tzplatform_config.h>
#include <system_info.h>

#include "syspopup_parser_db.h"
#include "syspopup_private.h"
#include "syspopup_type.h"

#define CREATE_SYSPOPUP_INFO_TABLE                                             \
	"PRAGMA journal_mode = PERSIST; "                                      \
	"CREATE TABLE IF NOT EXISTS syspopup_info ( "                          \
	"id INTEGER PRIMARY KEY, "                                             \
	"name TEXT UNIQUE NOT NULL, "                                          \
	"prio INTEGER, "                                                       \
	"focus INTEGER, "                                                      \
	"timeout INTEGER, "                                                    \
	"term_act INTEGER, "                                                   \
	"endkey_act INTEGER, "                                                 \
	"pkgname TEXT NOT NULL);"

#define QUERY_INSERT_SYSPOPUP_INFO                                             \
	"INSERT OR REPLACE INTO syspopup_info "                                \
	"(name, prio, focus, timeout, term_act, endkey_act, pkgname) "         \
	"VALUES(?, ?, ?, ?, ?, ?, ?);"

#define BIND_TEXT(db, stmt, i, text) do {                                      \
	if (sqlite3_bind_text(stmt, i, text, -1, SQLITE_STATIC) != SQLITE_OK) {\
		_E("sqlite3_bind_text() is failed. error(%s)",                 \
				sqlite3_errmsg(db));                           \
		sqlite3_finalize(stmt);                                        \
		return -1;                                                     \
	}                                                                      \
} while (0)

#define BIND_INT(db, stmt, i, val) do {                                        \
	if (sqlite3_bind_int(stmt, i, val) != SQLITE_OK) {                     \
		_E("sqlite3_bind_int() is failed. error(%s)",                  \
				sqlite3_errmsg(db));                           \
		sqlite3_finalize(stmt);                                        \
		return -1;                                                     \
	}                                                                      \
} while (0)

#define PATH_DB tzplatform_mkpath(TZ_SYS_DB, ".syspopup.db")
#define PATH_DB_JOURNAL tzplatform_mkpath(TZ_SYS_DB, ".syspopup.db-journal")
#define DB_LABEL "User::Home"

static sqlite3 *__db;

bool syspopup_parser_db_exist(void)
{
	if (access(PATH_DB, F_OK) == 0)
		return true;

	return false;
}

int syspopup_parser_db_unlink(void)
{
	int ret;

	ret = access(PATH_DB, F_OK);
	if (ret == 0) {
		ret = unlink(PATH_DB);
		if (ret != 0) {
			_E("Failed to delete %s. errno(%d)",
					PATH_DB, errno);
			return -1;
		}
	}

	return 0;
}

void syspopup_parser_db_close(void)
{
	if (__db == NULL)
		return;

	sqlite3_close(__db);
	__db = NULL;
}

int syspopup_parser_db_open(void)
{
	int ret;
	const char *query = CREATE_SYSPOPUP_INFO_TABLE;
	char *err_msg = NULL;

	ret = sqlite3_open_v2(PATH_DB, &__db,
			SQLITE_OPEN_CREATE | SQLITE_OPEN_READWRITE, NULL);
	if (ret != SQLITE_OK) {
		_E("Failed to create syspopup db - ret(%d)", ret);
		syspopup_parser_db_close();
		return -1;
	}

	ret = sqlite3_exec(__db, query, NULL, NULL, &err_msg);
	if (ret != SQLITE_OK) {
		_E("Failed to create syspopup info table(%s)", err_msg);
		sqlite3_free(err_msg);
		syspopup_parser_db_close();
		return -1;
	}

	ret = smack_setlabel(PATH_DB, DB_LABEL, SMACK_LABEL_ACCESS);
	if (ret < 0) {
		_E("Failed to set label");
		syspopup_parser_db_close();
		return -1;
	}

	if (access(PATH_DB_JOURNAL, F_OK) == 0) {
		ret = smack_setlabel(PATH_DB_JOURNAL, DB_LABEL,
				SMACK_LABEL_ACCESS);
		if (ret < 0) {
			_E("Failed to set label of journal file");
			syspopup_parser_db_close();
			return -1;
		}
	}

	return 0;
}

int syspopup_parser_db_insert(GList *syspopup_list)
{
	const char *query = QUERY_INSERT_SYSPOPUP_INFO;
	sqlite3_stmt *stmt = NULL;
	syspopup_t *info;
	GList *iter;
	int ret;
	int idx;

	if (!syspopup_list) {
		_E("Invalid parameter");
		return -1;
	}

	ret = sqlite3_prepare_v2(__db, query, strlen(query), &stmt, NULL);
	if (ret != SQLITE_OK) {
		_E("sqlite3_prepare_v2() is failed. error(%s)",
				sqlite3_errmsg(__db));
		return -1;
	}

	iter = syspopup_list;
	while (iter) {
		info = (syspopup_t *)iter->data;
		iter = g_list_next(iter);
		if (!info)
			continue;

		idx = 1;
		BIND_TEXT(__db, stmt, idx++, info->name);
		BIND_INT(__db, stmt, idx++, info->priority);
		BIND_INT(__db, stmt, idx++, info->focus);
		BIND_INT(__db, stmt, idx++, info->timeout);
		BIND_INT(__db, stmt, idx++, info->term_action);
		BIND_INT(__db, stmt, idx++, info->endkey_action);
		BIND_TEXT(__db, stmt, idx++, info->appid);

		ret = sqlite3_step(stmt);
		if (ret != SQLITE_DONE) {
			_E("sqlite3_step() is failed. name[%s] appid[%s]",
					info->name, info->appid);
			ret = -1;
			goto end;
		}

		sqlite3_reset(stmt);

		fprintf(stderr, "syspopup name[%s] appid[%s] priority[%d] "
				"focus[%d] timeout[%d] term-action[%d] "
				"endkey-action[%d]\n",
				info->name, info->appid, info->priority,
				info->focus, info->timeout, info->term_action,
				info->endkey_action);
	}

	ret = 0;
end:
	sqlite3_finalize(stmt);

	return ret;
}
